/************************************************
 *	FICHERO:	Gyro.h
 *
 *	AUTORES:	-Guillermo Maroto
 *
 *    ---------------+---------------                                                
 *              ___ /^^[___              _                                           
 *             /|^+----+   |#___________//                             
 *           ( -+ |____|    ______-----+/                                            
 *            ==_________--'            \                                            
 *               ~_|___|__             
 *
 *********************************************/
#pragma once

#include "MarotoCopter.h"
#include "WProgram.h"
#include "Sensor.h"

class Gyro: public Sensor 
{

protected:
  float gyroFullScaleOutput;
  float gyroScaleFactor;
  float smoothFactor;
  int gyroChannel[3];
  float gyroData[3];
  int gyroZero[3];
  int gyroADC[3];
  byte rollChannel, pitchChannel, yawChannel;
  int sign[3];
  float rawHeading, gyroHeading;
  long int previousGyroTime;
  //unsigned long currentTime, previousTime; // AKA - Changed to remove HONKS time smoothing
	
public:	
	Gyro(void)
	{
		sign[ROLL] = -1;
		sign[PITCH] = 1;
		sign[YAW] = 1;
	}

	virtual void initialize(void) = 0;
	
	virtual void measure(void) = 0;
	
	virtual void calibrate(void) = 0;
	
	virtual void autoZero(void){};
	
	virtual const int getFlightData(byte);	
	
  // The following functions are common between all Gyro subclasses
  void _initialize(byte rollChannel, byte pitchChannel, byte yawChannel) {
    gyroChannel[ROLL] = rollChannel;
    gyroChannel[PITCH] = pitchChannel;
    gyroChannel[ZAXIS] = yawChannel;
    
    gyroZero[ROLL] = readFloat(GYRO_ROLL_ZERO_ADR);
    gyroZero[PITCH] = readFloat(GYRO_PITCH_ZERO_ADR);
    gyroZero[ZAXIS] = readFloat(GYRO_YAW_ZERO_ADR);
    smoothFactor = readFloat(GYROSMOOTH_ADR);
  }
	
	
  // returns the raw ADC value from the gyro, with sign change if needed, not smoothed or scaled to SI units    
  const int getRaw(byte axis) {
    return gyroADC[axis] * sign[axis];
  }
  
  // returns the smoothed and scaled to SI units value of the Gyro with sign change if needed
  // centered on zero radians +/-
  const float getData(byte axis) {
    return gyroData[axis] * sign[axis];
  }
  
  //  inverts, if needed the sign on the specific axis
  const int invert(byte axis) {
    sign[axis] = -sign[axis];
    return sign[axis];
  }
  
  const int getZero(byte axis) {
    return gyroZero[axis];
  }
  
  void setZero(byte axis, int value) {
    gyroZero[axis] = value;
  }    
  
  // returns the scale factor used for SI units on the gyro
  const float getScaleFactor() {
    return gyroScaleFactor;
  }

  // returns the smooth factor used on the gyro
  const float getSmoothFactor(void) {
    return smoothFactor;
  }
  
  void setSmoothFactor(float value) {
    smoothFactor = value;
  }
  
  // returns gyro based heading as +/- PI in radians
  const float getHeading(void) {
    //div_t integerDivide;
    
    //integerDivide = div(rawHeading, 2*PI);
    gyroHeading = rawHeading; // + (integerDivide.quot * -(2*PI));
    if (gyroHeading > PI) gyroHeading -= (2*PI);
    if (gyroHeading < -PI) gyroHeading += (2*PI);
    //Serial.print(integerDivide.quot);Serial.print(",");Serial.print(integerDivide.rem);Serial.println();
    return gyroHeading;
  } 
};
