/***********************************************
 *	FILE:	Gyro_ITG3200.cpp
 *
 *	AUTHORS:	-Guillermo Maroto
 *
 *	Based on Aeroquad.com - Gyro.h         
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/

#include "Gyro_ITG3200.h"


void Gyro_ITG3200::initialize(void) {
    this->_initialize(0,1,2);
    
    //gyroLastData = 0.0;  // initalize for rawHeading, may be able to be removed in the future
    
    // Check if gyro is connected
 
	if (readWhoI2C(gyroAddress) != gyroAddress)  // hardcoded for +1 of address specific to sparkfun 6dof imu

    Serial.println("Gyro not found!");
	
    // Thanks to SwiftingSpeed for updates on these settings
    // http://aeroquad.com/showthread.php?991-AeroQuad-Flight-Software-v2.0&p=11207&viewfull=1#post11207
    updateRegisterI2C(gyroAddress, 0x3E, 0x80); // send a reset to the device
    updateRegisterI2C(gyroAddress, 0x16, 0x1D); // 10Hz low pass filter
    updateRegisterI2C(gyroAddress, 0x3E, 0x01); // use internal oscillator 
}

void Gyro_ITG3200::measure(void) {
    sendByteI2C(gyroAddress, 0x1D);
    Wire.requestFrom(gyroAddress, 6);
	
	
	//Modified by Maroto
	
	gyroADC[PITCH] = ((Wire.receive() << 8) | Wire.receive()) - gyroZero[PITCH];
	gyroADC[ROLL] = gyroZero[ROLL] - ((Wire.receive() << 8) | Wire.receive());
	gyroADC[YAW] = gyroZero[YAW] - ((Wire.receive() << 8) | Wire.receive());
	
	gyroData[PITCH] = filterSmooth((float)gyroADC[PITCH] * gyroScaleFactor, gyroData[PITCH], smoothFactor);
	gyroData[ROLL] = filterSmooth((float)gyroADC[ROLL] * gyroScaleFactor, gyroData[ROLL], smoothFactor);
	gyroData[YAW] = filterSmooth((float)gyroADC[YAW] * gyroScaleFactor, gyroData[YAW], smoothFactor);
	
	/*
    for (byte axis = ROLL; axis < LASTAXIS; axis++) {
		if (axis == ROLL)
			gyroADC[axis] = ((Wire.receive() << 8) | Wire.receive()) - gyroZero[axis];
		else
			gyroADC[axis] = gyroZero[axis] - ((Wire.receive() << 8) | Wire.receive());
		gyroData[axis] = filterSmooth((float)gyroADC[axis] * gyroScaleFactor, gyroData[axis], smoothFactor);
    }
	*/
	
    //calculateHeading();
    // gyroLastADC can maybe replaced with Zero, but will leave as is for now
    // this provides a small guard band for the gyro on Yaw before it increments or decrements the rawHeading 
    long int currentGyroTime = micros();
    if (gyroData[YAW] > radians(1.0) || gyroData[YAW] < radians(-1.0)) {
		rawHeading += gyroData[YAW] * ((currentGyroTime - previousGyroTime) / 1000000.0);
    }
    previousGyroTime = currentGyroTime;
	
}

void Gyro_ITG3200::calibrate() {
    autoZero();
    writeFloat(gyroZero[ROLL], GYRO_ROLL_ZERO_ADR);
    writeFloat(gyroZero[PITCH], GYRO_PITCH_ZERO_ADR);
    writeFloat(gyroZero[YAW], GYRO_YAW_ZERO_ADR);
	
	Serial.println("Gyro_ITG3200 calibrated!");
}

// returns raw ADC data from the Gyro centered on zero +/- values
const int Gyro_ITG3200::getFlightData(byte axis){
    //int reducedData = getRaw(axis) >> 3;
    //if ((reducedData < 5) && (reducedData > -5)) reducedData = 0;
    if (axis == PITCH)
		return (getRaw(axis) >> 3);	//return -(getRaw(axis) >> 3);
    else
		return (getRaw(axis) >> 3);
}

void Gyro_ITG3200::autoZero() {
    int findZero[FINDZERO];
    for (byte calAxis = ROLL; calAxis < LASTAXIS; calAxis++) {
		for (int i=0; i<FINDZERO; i++) {
			sendByteI2C(gyroAddress, (calAxis * 2) + 0x1D);
			findZero[i] = readWordI2C(gyroAddress);
			delay(10);
		}
		
		//Modified by Maroto
		if(calAxis == ROLL)
			gyroZero[PITCH] = findMedian(findZero, FINDZERO);
		else if(calAxis == PITCH)
			gyroZero[ROLL] = findMedian(findZero, FINDZERO);
		else
			gyroZero[calAxis] = findMedian(findZero, FINDZERO);
    }
}