//#include <stdlib.h>
//#include <math.h>
#include "WProgram.h"
#include "pins_arduino.h"

/****************************************************************************
 *********************  User Definition Section *****************************
 ****************************************************************************/

//#define BAUD 115200
#define BAUD 111111 // use this to be compatible with USB and XBee connections
//#define BAUD 57600


//PID parameters

#define	PROLL	50
#define IROLL	0
#define DROLL	-600

#define	PROLLATTITUDE	5
#define IROLLATTITUDE	1
#define DROLLATTITUDE	0

#define PPITCH	50
#define IPITCH	0
#define DPITCH	-600

#define PPITCHATTITUDE	5
#define IPITCHATTITUDE	1
#define DPITCHATTITUDE	0

#define PYAW	200
#define IYAW	5
#define DYAW	0

#define PHEADING 3
#define IHEADING 0.1
#define DHEADING 0


//Smooth parameters

#define GYRO_SMOOTH_FACTOR 1.0
#define ACCEL_SMOOTH_FACTOR 1.0

//Recevier parameters

#define MAXRECEIVERTHROTTLE 1500	//Limita el rango del acelerador de la emisora RC de 1000 a MAXRECEIVERTHROTTLE,
									//con esto conseguimos mayor sensibilidad

/****************************************************************************
 ********************* End of User Definition Section ***********************
 ****************************************************************************/

#define PI 3.141592654
#define	GRARAD 360/(2*PI) // grados/Radianes  paso de Grados a Radianes

#define LEDPIN 13
#define ON 1
#define OFF 0

#define LED2PIN 4
#define LED3PIN 31

// Basic axis definitions
#define ROLL 0
#define PITCH 1
#define YAW 2
#define THROTTLE 3
#define MODE 4
#define AUX 5
#define AUX2 6
#define AUX3 7
#define XAXIS 0
#define YAXIS 1
#define ZAXIS 2
#define LASTAXIS 3
#define LEVELROLL 3
#define LEVELPITCH 4
#define LASTLEVELAXIS 5
#define HEADING 5
#define LEVELGYROROLL 6
#define LEVELGYROPITCH 7


// Smoothing filter parameters
#define GYRO 0
#define ACCEL 1
#define FINDZERO 49

// Sensor pin assignments
#define PITCHACCELPIN 0
#define ROLLACCELPIN 1
#define ZACCELPIN 2
#define PITCHRATEPIN 3
#define ROLLRATEPIN 4
#define YAWRATEPIN 5
#define AZPIN 12 // Auto zero pin for IDG500 gyros

// Motor control variables
#define FRONT 0
#define REAR 1
#define RIGHT 2
#define LEFT 3
#define MOTORID1 0		
#define MOTORID2 1		
#define MOTORID3 2		
#define MOTORID4 3		
#define MOTORID5 4		
#define MOTORID6 5
#define MINCOMMAND 1000
#define MEDCOMMAND 1100
#define MAXCOMMAND 1800 // 1800 por seguridad// 2000
#define LASTMOTOR 4


#define TRIMOTORFRONT	0
#define TRIMOTORREAR	0
#define TRIMOTORLEFT	0
#define TRIMOTORRIGHT	0


#define AILOOPTIME			2
#define CONTROLLOOPTIME		2
#define TELEMETRYLOOPTIME	0
#define RECEIVERLOOPTIME	100

// Receiver variables
#define TIMEOUT 25000
#define MINCOMMAND 1000
#define MIDCOMMAND 1500
#define MAXCOMMAND 2000
#define MINDELTA 200
#define MINCHECK MINCOMMAND + 100
#define MAXCHECK MAXCOMMAND - 100
#define MINTHROTTLE MINCOMMAND + 100
#define LEVELOFF 100
//int delta;

#define PWM2RAD 0.002 //  Based upon 5RAD for full stick movement, you take this times the RAD to get the PWM conversion factor
#define LASTCHANNEL 6
#define ATTITUDE_SCALING (0.55 * PWM2RAD)


#define RISING_EDGE 1
#define FALLING_EDGE 0
#define MINONWIDTH 950
#define MAXONWIDTH 2075
#define MINOFFWIDTH 12000
#define MAXOFFWIDTH 24000

// **************************************************************
// *************************** EEPROM ***************************
// **************************************************************
// EEPROM storage addresses
#define ROLL_PID_GAIN_ADR 0
#define LEVELROLL_PID_GAIN_ADR 12
#define YAW_PID_GAIN_ADR 24
#define WINDUPGUARD_ADR 36
#define LEVELLIMIT_ADR 40
#define LEVELOFF_ADR 44
#define XMITFACTOR_ADR 48
#define GYROSMOOTH_ADR 52
#define ACCSMOOTH_ADR 56
#define LEVELPITCHCAL_ADR 60
#define LEVELROLLCAL_ADR 64
#define LEVELZCAL_ADR 68
#define FILTERTERM_ADR 72
#define NVM_TRANSMITTER_SCALE_OFFSET_SMOOTH 76  // needs 8 channel with 3 entries of float (4 byte) -> 96 byte
#define PITCH_PID_GAIN_ADR 172
#define LEVELPITCH_PID_GAIN_ADR 184
#define HEADINGSMOOTH_ADR 200
#define HEADING_PID_GAIN_ADR 204
#define AREF_ADR 216
#define FLIGHTMODE_ADR 220
#define LEVEL_GYRO_ROLL_PID_GAIN_ADR 224
#define LEVEL_GYRO_PITCH_PID_GAIN_ADR 236
#define HEADINGHOLD_ADR 248
#define MINACRO_ADR 252
#define ACCEL1G_ADR 256
#define ALTITUDE_PGAIN_ADR 260
#define ALTITUDE_IGAIN_ADR 264
#define ALTITUDE_DGAIN_ADR 268
#define ALTITUDE_MAX_THROTTLE_ADR 272
#define ALTITUDE_MIN_THROTTLE_ADR 276
#define ALTITUDE_SMOOTH_ADR 280
#define ZDAMP_PGAIN_ADR 284
#define ZDAMP_IGAIN_ADR 288
#define ZDAMP_DGAIN_ADR 292
#define ALTITUDE_WINDUP_ADR 296
#define MAGXMAX_ADR 300
#define MAGXMIN_ADR 304
#define MAGYMAX_ADR 308
#define MAGYMIN_ADR 312
#define MAGZMAX_ADR 316
#define MAGZMIN_ADR 320
#define MCAMERAPITCH_ADR 324
#define MCAMERAROLL_ADR 328
#define MCAMERAYAW_ADR 332
#define CENTERPITCH_ADR 336
#define CENTERROLL_ADR 340
#define CENTERYAW_ADR 344
#define SERVOMINPITCH_ADR 348
#define SERVOMINROLL_ADR 352
#define SERVOMINYAW_ADR 356
#define SERVOMAXPITCH_ADR 360
#define SERVOMAXROLL_ADR 364
#define SERVOMAXYAW_ADR 368
#define GYRO_ROLL_ZERO_ADR 372
#define GYRO_PITCH_ZERO_ADR 376
#define GYRO_YAW_ZERO_ADR 380
