/***********************************************
 *	FILE:	MarotoCopter.pde
 *
 *	AUTHORS:	-Guillermo Maroto
 *
 *           
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/

#include <Wire.h>
#include <EEPROM.h>
#include "MarotoCopter.h"
//#include "DataStorage.h"

#include "Quadcopter.h"
Quadcopter quadcopter;

bool sensorLoop =		1;
bool controlLoop =		1;
bool ComunicationLoop =	1;
bool receiverLoop =		1;


// Flight Mode
unsigned long frameCounter = 0; // main loop executive frame counter

/**************************************************************/
/******************* Loop timing parameters *******************/
/**************************************************************/

float G_Dt = 0.002;
// Offset starting times so that events don't happen at the same time
// main loop times
unsigned long previousTime = 0;
unsigned long currentTime = 0;
unsigned long deltaTime = 0;
// sub loop times
unsigned long oneHZpreviousTime;
unsigned long tenHZpreviousTime;
unsigned long twentyFiveHZpreviousTime;
unsigned long fiftyHZpreviousTime;
unsigned long hundredHZpreviousTime;


/********************     Setup Routines  ********************/

void setup() 
{
	quadcopter.initialize();
	
	quadcopter.initializeComunication();
	
	quadcopter.initializeReceiver();
	
	pinMode(LEDPIN, OUTPUT);
	digitalWrite(LEDPIN, LOW);
	
	pinMode(LED3PIN, OUTPUT);

		
	quadcopter.initializeState();
	
	quadcopter.initializeControl();

	quadcopter.initializeMotors();
	
	
	previousTime = micros();
	digitalWrite(LEDPIN, HIGH);

}

/********************     Loop  ********************/
void loop ()
{
  // Measure loop rate
  currentTime = micros();
  quadcopter.setCurrentTime(currentTime);
  deltaTime = currentTime - previousTime;

  // Main scheduler loop set for 100hz
  if (deltaTime >= 10000) {
  
	
    frameCounter++;
    
    // ================================================================
    // 100hz task loop
    // ================================================================
    if (frameCounter %   1 == 0) {  //  100 Hz tasks
	
		G_Dt = (currentTime - hundredHZpreviousTime) / 1000000.0;
		hundredHZpreviousTime = currentTime;
		
		quadcopter.flightAngle.G_Dt=G_Dt;//CHAPU
		
		// Measures sensor data and calculates attitude
		if (sensorLoop == 1)
		{
			// measure critical sensors
			quadcopter.measureState();		
		} 
	
		// Combines external pilot commands and measured sensor data to generate motor commands
		if (controlLoop == 1)
		{ 
			quadcopter.processFlightControl();
		} 
	
	}
	
    // ================================================================
    // 50hz task loop
    // ================================================================
    if (frameCounter %   2 == 0) {  //  50 Hz tasks
      
      G_Dt = (currentTime - fiftyHZpreviousTime) / 1000000.0;
      fiftyHZpreviousTime = currentTime;
	
		if (receiverLoop == 1) 
		{ 
			quadcopter.readReceiver();
		}

    }
	
    // ================================================================
    // 25hz task loop
    // ================================================================
//    if (frameCounter %   4 == 0) {  //  25 Hz tasks
//      
//      G_Dt = (currentTime - twentyFiveHZpreviousTime) / 1000000.0;
//      twentyFiveHZpreviousTime = currentTime;
//      
//      if (sensorLoop == ON) {
//
//      }
//
//    }
        // ================================================================
    // 10hz task loop
    // ================================================================
    if (frameCounter %  10 == 0) {  //   10 Hz tasks

      
      G_Dt = (currentTime - tenHZpreviousTime) / 1000000.0;
      tenHZpreviousTime = currentTime;

      // Listen for configuration commands and reports telemetry
      if (ComunicationLoop == ON) {
		quadcopter.readComunication();
		quadcopter.writeComunication();
      }
	}
      
	previousTime = currentTime;
  }
  
  if (frameCounter >= 100) 
      frameCounter = 0;


}



