/***********************************************
 *	FILE:	Receiver.h
 *
 *	AUTHORS:	-Guillermo Maroto
 *	Based on Aeroquad.com/Receiver.h
 *           
 *   --------+--------         --------+--------
 *          | |_______________________| | 
 *                   |         |
 *                   |_________|
 *                 ~____|___|____    
 **********************************************/
#pragma once

#include "MarotoCopter.h"


class  Receiver {
public:
	int receiverData[LASTCHANNEL];
	int transmitterCommand[LASTCHANNEL];
	int transmitterCommandSmooth[LASTCHANNEL];
	int transmitterZero[3];
	int transmitterTrim[3];
	// Controls the strength of the commands sent from the transmitter
	// xmitFactor ranges from 0.01 - 1.0 (0.01 = weakest, 1.0 - strongest)
	float xmitFactor; // Read in from EEPROM
	float transmitterSmooth[LASTCHANNEL];
	float mTransmitter[LASTCHANNEL];
	float bTransmitter[LASTCHANNEL];
	//unsigned long currentTime, previousTime;
	
	Receiver(void) {
		transmitterCommand[ROLL] = 1500;
		transmitterCommand[PITCH] = 1500;
		transmitterCommand[YAW] = 1500;
		transmitterCommand[THROTTLE] = 1000;
		transmitterCommand[MODE] = 1000;
		transmitterCommand[AUX] = 1000;
		
		for (byte channel = ROLL; channel < LASTCHANNEL; channel++)
			transmitterCommandSmooth[channel] = 1.0;
		for (byte channel = ROLL; channel < THROTTLE; channel++)
			transmitterZero[channel] = 1500;
	}
	
	// ******************************************************************
	// The following function calls must be defined in any new subclasses
	// ******************************************************************
	virtual void initialize(void);
	virtual void read(void);
	
	// **************************************************************
	// The following functions are common between all Gyro subclasses
	// **************************************************************
	
	void _initialize(void);
	
	// returns non-smoothed non-scaled ADC data in PWM full range 1000-2000 values
	const int getRaw(byte channel) {
		return receiverData[channel];
	}
	
	// returns raw but smoothed receiver(channel) in PWM
	const int getRawSmoothed(byte channel) {
		return transmitterCommandSmooth[channel];
	}
	
	// returns smoothed & scaled receiver(channel) in PWM values, zero centered
	const int getData(byte channel) {
		return transmitterCommand[channel];
	}
	
	// return the smoothed & scaled number of radians/sec in stick movement - zero centered
	const float getSIData(byte channel) {
		// 2.3 Original
		return ((transmitterCommand[channel] - transmitterZero[channel]) * (2.5 * PWM2RAD));  // +/- 2.5RPS 50% of full rate
		// 2.3 Stable
		//return ((transmitterCommand[channel] - transmitterZero[channel]) * (5.0 * PWM2RAD));  // +/- 5RPS factored by xmitfactor of full rate
	}
	
	const int getTrimData(byte channel) {
		return receiverData[channel] - transmitterTrim[channel];
	}
	
	// returns Zero value of channel in PWM
	const int getZero(byte channel) {
		return transmitterZero[channel];
	}
	// sets zero value of channel in PWM
	void setZero(byte channel, int value) {
		transmitterZero[channel] = value;
	}
	
	const int getTransmitterTrim(byte channel) {
		return transmitterTrim[channel];
	}
	
	void setTransmitterTrim(byte channel, int value) {
		transmitterTrim[channel] = value;
	}
	
	const float getSmoothFactor(byte channel) {
		return transmitterSmooth[channel];
	}
	
	void setSmoothFactor(byte channel, float value) {
		transmitterSmooth[channel] = value;
	}
	
	const float getXmitFactor(void) {
		return xmitFactor;
	}
	
	void setXmitFactor(float value) {
		xmitFactor = value;
	}
	
	const float getTransmitterSlope(byte channel) {
		return mTransmitter[channel];
	}
	
	void setTransmitterSlope(byte channel, float value) {
		mTransmitter[channel] = value;
	}
	
	const float getTransmitterOffset(byte channel) {
		return bTransmitter[channel];
	}
	
	void setTransmitterOffset(byte channel, float value) {
		bTransmitter[channel] = value;
	}
	
	const float getAngle(byte channel) {
		// Scale 1000-2000 usecs to -45 to 45 degrees
		// m = 0.09, b = -135
		return (0.09 * transmitterCommand[channel]) - 135;
		//return (0.09 * receiverData[channel]) - 135;
	}
};